/*
 * asteriskaudio.hxx
 *
 * Sound driver implementation for ASTERISK PBX.
 * OpenH323 Wrapper Library.
 *
 * Copyright (c) 2002,2003 InAccess Networks
 * Michalis Manousos <manousos@inaccessnetworks.com>
 * Dimitris Economou <decon@inaccessnetworks.com>
 *
 * This file is part of "H323 support for ASTERISK"
 *
 * "H323 support for ASTERISK" is free software;
 * you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation;
 * either version 2 of the License, or (at your option) any later version. 
 *
 * "H323 support for ASTERISK" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. 
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 * $Id: asteriskaudio.hxx,v 1.14 2003/03/05 12:37:36 manousos Exp $
 *
 */

///////////////////////////////////////////////////////////////////////////////

#ifndef	_ASTERISKAUDIO_HXX
#define	_ASTERISKAUDIO_HXX

#ifdef	WRAPTRACING

// Set a default tracing level
#ifndef	WRAPTRACING_LEVEL
#define	WRAPTRACING_LEVEL	5
#endif

#define	WRAPTRACE(level, args) \
	if (level >= WRAPTRACING_LEVEL) \
			cout << (this->Class()) << "::" << __FUNCTION__ << ": " << args << endl
#define	WRAPTRACEAPI(level, args) \
	if (level >= WRAPTRACING_LEVEL) \
			cout << "Wrapper API::" << __FUNCTION__ << ": " << args << endl

#else	// WRAPTRACING

#define	WRAPTRACE(level, args)
#define	WRAPTRACEAPI(level, args)

#endif	// WRAPTRACING


/**
 * PAsteriskAudioDelay class
 * This class implements a delay and is used by
 * the "Write" member function of the PChannel descentant.
 */
class PAsteriskAudioDelay : public PObject {

	PCLASSINFO(PAsteriskAudioDelay, PObject);

	public:

	PAsteriskAudioDelay();
	~PAsteriskAudioDelay();
	BOOL Delay(int time);
	void Restart();
	int  GetError();

	protected:

	PTime  previousTime;
	BOOL   firstTime;
	int    error;

};


/**
 * PAsteriskSoundChannel class
 * This class interfaces the ASTERISK PBX 
 * (open, close, read, write operations)
 */
class PAsteriskSoundChannel : public PSoundChannel {

	PCLASSINFO(PAsteriskSoundChannel, PSoundChannel);

	public:

	PAsteriskSoundChannel();
	~PAsteriskSoundChannel();
	void Construct();
	static PStringArray GetDeviceNames(Directions);
	static PString GetDefaultDevice(Directions);
	BOOL Open(const PString &, int, Directions, unsigned, unsigned, unsigned);
	BOOL Close();
	BOOL SetFormat(unsigned, unsigned, unsigned);
	BOOL SetBuffers(PINDEX, PINDEX);
	BOOL GetBuffers(PINDEX &, PINDEX &);
	BOOL Write(const void *, PINDEX);
	PINDEX GetLastWriteCount() const;
	BOOL PlaySound(const PSound &, BOOL);
	BOOL PlayFile(const PFilePath &, BOOL);
	BOOL HasPlayCompleted();
	BOOL WaitForPlayCompletion();
	BOOL Read(void *, PINDEX);
	BOOL RecordSound(PSound &);
	BOOL RecordFile(const PFilePath &);
	BOOL StartRecording();
	BOOL IsRecordBufferFull();
	BOOL AreAllRecordBuffersFull();
	BOOL WaitForRecordBufferFull();
	BOOL WaitForAllRecordBuffersFull();
	BOOL Abort();

	protected:

	PAsteriskAudioDelay	astWriteDelay;
	PAsteriskAudioDelay	astReadDelay;
	PINDEX bufSize, bufCount;
	unsigned mediaFormat;
	unsigned frameTime;
	int mutex_flag;
	PINDEX lastWritten;

};

#endif	// _ASTERISKAUDIO_HXX
// End of file //////////////////////////////////////////////////////////////
